import pytest
from django.urls import reverse
from model_bakery import baker
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions
from selenium.webdriver.support.wait import WebDriverWait
from splinter.driver.webdriver.firefox import WebDriver

from bpp.models import Wydawnictwo_Ciagle, Wydawnictwo_Zwarte
from bpp.tests import fill_admin_inline, normalize_html

from django_bpp.selenium_util import SHORT_WAIT_TIME, wait_for_page_load


@pytest.mark.parametrize("klass", ["wydawnictwo_ciagle", "wydawnictwo_zwarte"])
def test_changelist_no_argument(klass, live_server, admin_browser: WebDriver):
    url = f"admin:bpp_{klass}_autor_changelist"
    admin_browser.visit(live_server + reverse(url))
    assert "Musisz wejść w edycję" in normalize_html(admin_browser.html)


@pytest.mark.parametrize("klass", ["wydawnictwo_ciagle", "wydawnictwo_zwarte"])
def test_edit_btn_invisible(klass, live_server, admin_browser: WebDriver):
    # Przy dodawaniu publikacji -- brak przycisku "edytuj autorów"
    url = f"admin:bpp_{klass}_add"
    with wait_for_page_load(admin_browser):
        admin_browser.visit(live_server + reverse(url))
    assert "Edytuj autorów" not in normalize_html(admin_browser.html)


@pytest.mark.parametrize(
    "klass,model",
    [
        ("wydawnictwo_ciagle", Wydawnictwo_Ciagle),
        ("wydawnictwo_zwarte", Wydawnictwo_Zwarte),
    ],
)
def test_edit_btn_appears(klass, model, live_server, admin_browser: WebDriver):
    # Przy edytowaniu publikacji -- przycisk "edytuj autorów" jest
    res = baker.make(model)

    url = f"admin:bpp_{klass}_change"
    with wait_for_page_load(admin_browser):
        admin_browser.visit(live_server + reverse(url, args=(res.pk,)))

    baker.make(model)

    assert "Autorzy" in normalize_html(admin_browser.html)


@pytest.mark.parametrize(
    "klass,model",
    [
        ("wydawnictwo_ciagle", Wydawnictwo_Ciagle),
        ("wydawnictwo_zwarte", Wydawnictwo_Zwarte),
    ],
)
def test_changelist_with_argument(klass, model, admin_browser, live_server):
    rec = baker.make(model)
    url = f"admin:bpp_{klass}_autor_changelist"
    with wait_for_page_load(admin_browser):
        admin_browser.visit(live_server + reverse(url) + f"?rekord__id__exact={rec.pk}")

    assert "Dodaj powiązanie" in normalize_html(admin_browser.html)

    # wchodzimy z okreslonym ID, czy jest lista autorów?
    with wait_for_page_load(admin_browser):
        admin_browser.find_by_text("Edycja rekordu").click()

    assert "Autorzy" in normalize_html(admin_browser.html)


@pytest.mark.parametrize(
    "klass,model",
    [
        ("wydawnictwo_ciagle", Wydawnictwo_Ciagle),
        ("wydawnictwo_zwarte", Wydawnictwo_Zwarte),
    ],
)
def test_changeform_save(klass, model, admin_browser, live_server):
    rec = baker.make(model)
    wa = baker.make(model.autorzy.through, rekord=rec)

    url = f"admin:bpp_{klass}_autor_change"

    with wait_for_page_load(admin_browser):
        admin_browser.visit(live_server + reverse(url, args=(wa.pk,)))

    WebDriverWait(admin_browser.driver, SHORT_WAIT_TIME).until(
        expected_conditions.presence_of_element_located((By.NAME, "_save"))
    )

    # wchodzimy z okreslonym ID na okreslony rekord, zapisujmey, czy jest OK ?
    with wait_for_page_load(admin_browser):
        admin_browser.find_by_name("_save").click()

    assert "Dodaj powiązanie" in normalize_html(admin_browser.html)


@pytest.mark.parametrize(
    "klass,model",
    [
        ("wydawnictwo_ciagle", Wydawnictwo_Ciagle),
        ("wydawnictwo_zwarte", Wydawnictwo_Zwarte),
    ],
)
def test_changeform_add(
    klass,
    model,
    live_server,
    admin_browser,
    autor_jan_nowak,
    jednostka,
    typy_odpowiedzialnosci,
):
    assert model.autorzy.through.objects.count() == 0

    rec = baker.make(model)
    url = f"admin:bpp_{klass}_autor_changelist"
    admin_browser.visit(live_server + reverse(url) + f"?rekord__id__exact={rec.pk}")

    with wait_for_page_load(admin_browser):
        admin_browser.find_by_name("_add_wa").click()

    fill_admin_inline(admin_browser, autor_jan_nowak, jednostka, prefix="id_")

    with wait_for_page_load(admin_browser):
        admin_browser.find_by_name("_save").click()

    assert "Edycja rekordu" in normalize_html(admin_browser.html)

    assert model.autorzy.through.objects.count() == 1



import messages_extends as messages
from django.core.management import BaseCommand

from notifications.core import get_channel_name_for_user, send_notification

from django.contrib.auth import get_user_model


class Command(BaseCommand):
    help = "Wysyla notyfikacje realtime via django-channels"
    args = "<username> <message>"

    def add_arguments(self, parser):
        parser.add_argument("username")
        parser.add_argument("text")

    def handle(self, *args, **options):
        user = get_user_model().objects.get(username=options["username"])
        channel_name = get_channel_name_for_user(user)

        send_notification(
            channel_name,
            messages.INFO_PERSISTENT,
            options["text"],
        )



# -*- coding: utf-8 -*-


from django.db import migrations, models

def ustaw_typy(apps, schema_editor):
    Charakter_Formalny = apps.get_model("bpp", "Charakter_Formalny")
    for skrot in ['KS', 'KSZ', 'KSP', 'PA', 'SKR']:
        chf = Charakter_Formalny.objects.get(skrot=skrot)
        chf.ksiazka_pbn = True
        chf.save()
    for skrot in ['frg', 'ROZ', 'ROZS']:
        chf = Charakter_Formalny.objects.get(skrot=skrot)
        chf.rozdzial_pbn = True
        chf.save()

def noop(apps, schema_migration):
    pass


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0068_indeks_liczba_znakow'),
    ]

    operations = [
        migrations.RunPython(ustaw_typy, noop)
    ]



# -*- encoding: utf-8 -*-
import itertools

try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse
from django.db.models.aggregates import Sum
from django.template.defaultfilters import safe
from django.utils.functional import cached_property
from django_tables2 import Column
from django_tables2.export.views import ExportMixin
from django_tables2.tables import Table
from django_tables2.views import SingleTableView

from bpp.models import Autor, Sumy, OpcjaWyswietlaniaField, Uczelnia
from bpp.models.struktura import Wydzial


class RankingAutorowTable(Table):
    class Meta:
        attrs = {"class": "bpp-table"}
        model = Autor
        order_by = ("-impact_factor_sum", "autor__nazwisko")
        fields = (
            "lp",
            "autor",
            "impact_factor_sum",
            "liczba_cytowan_sum",
            "punkty_kbn_sum",
        )

    lp = Column(
        empty_values=(),
        orderable=False,
        attrs={"td": {"class": "bpp-lp-column"}},
        exclude_from_export=True,
    )

    autor = Column(order_by=("autor__nazwisko", "autor__imiona"))
    punkty_kbn_sum = Column("Punkty PK", "punkty_kbn_sum")
    impact_factor_sum = Column("Impact Factor", "impact_factor_sum")
    liczba_cytowan_sum = Column("Liczba cytowań", "liczba_cytowan_sum")

    def render_lp(self):
        self.lp_counter = getattr(
            self, "lp_counter", itertools.count(self.page.start_index())
        )
        return "%i." % next(self.lp_counter)

    def render_autor(self, record):
        return safe(
            '<a href="%s">%s</a>'
            % (
                reverse("bpp:browse_autor", args=(record.autor.slug,)),
                str(record.autor),
            )
        )

    def value_autor(self, record):
        return str(record.autor)


class RankingAutorowJednostkaWydzialTable(RankingAutorowTable):
    class Meta:
        fields = (
            "lp",
            "autor",
            "jednostka",
            "wydzial",
            "impact_factor_sum",
            "liczba_cytowan_sum",
            "punkty_kbn_sum",
        )
        order_by = ("-impact_factor_sum", "autor__nazwisko")

    jednostka = Column(accessor="jednostka.nazwa")
    wydzial = Column(accessor="jednostka.wydzial.nazwa")


class RankingAutorow(ExportMixin, SingleTableView):
    template_name = "raporty/ranking-autorow.html"

    def get_table_class(self):
        if self.rozbij_na_wydzialy:
            return RankingAutorowJednostkaWydzialTable
        return RankingAutorowTable

    @cached_property
    def rozbij_na_wydzialy(self):
        return self.request.GET.get("rozbij_na_jednostki", "True") == "True"

    @cached_property
    def tylko_afiliowane(self):
        return self.request.GET.get("tylko_afiliowane", "False") == "True"

    def get_queryset(self):
        qset = Sumy.objects.all()
        qset = qset.filter(
            rok__gte=self.kwargs["od_roku"], rok__lte=self.kwargs["do_roku"]
        )
        wydzialy = self.get_wydzialy()
        if wydzialy:
            qset = qset.filter(jednostka__wydzial__in=wydzialy)

        if self.tylko_afiliowane:
            qset = qset.filter(jednostka__skupia_pracownikow=True)
            qset = qset.filter(afiliuje=True)

        if self.rozbij_na_wydzialy:
            qset = qset.prefetch_related("jednostka__wydzial").select_related(
                "autor", "jednostka"
            )
            qset = qset.group_by("autor", "jednostka")
        else:
            qset = qset.select_related("autor")
            qset = qset.group_by("autor")

        qset = qset.annotate(
            impact_factor_sum=Sum("impact_factor"),
            liczba_cytowan_sum=Sum("liczba_cytowan"),
            punkty_kbn_sum=Sum("punkty_kbn"),
        )
        qset = qset.exclude(impact_factor_sum=0, liczba_cytowan_sum=0, punkty_kbn_sum=0)

        qset = qset.exclude(autor__pokazuj=False)

        uczelnia = Uczelnia.objects.get_default()
        if uczelnia is not None:
            ukryte_statusy = uczelnia.ukryte_statusy("rankingi")
            if ukryte_statusy:
                qset = qset.exclude(status_korekty_id__in=ukryte_statusy)

        return qset

    def get_dostepne_wydzialy(self):
        return Wydzial.objects.filter(zezwalaj_na_ranking_autorow=True)

    def get_wydzialy(self):
        base_query = self.get_dostepne_wydzialy()

        wydzialy = self.request.GET.getlist("wydzialy[]")
        if wydzialy:
            try:
                wydzialy = base_query.filter(pk__in=[int(x) for x in wydzialy])
                return wydzialy
            except (TypeError, ValueError):
                pass

        return base_query

    def get_context_data(self, **kwargs):
        context = super(SingleTableView, self).get_context_data(**kwargs)
        context["od_roku"] = self.kwargs["od_roku"]
        context["do_roku"] = self.kwargs["do_roku"]
        jeden_rok = False
        if self.kwargs["od_roku"] == self.kwargs["do_roku"]:
            context["rok"] = self.kwargs["od_roku"]
            jeden_rok = True

        wydzialy = self.get_wydzialy()
        context["wydzialy"] = wydzialy

        if jeden_rok:
            context["table_title"] = "Ranking autorów za rok %s" % context["rok"]
        else:
            context["table_title"] = "Ranking autorów za lata %s - %s" % (
                context["od_roku"],
                context["do_roku"],
            )
        context["tab_subtitle"] = ""
        if len(wydzialy) != len(self.get_dostepne_wydzialy()):
            context["table_subtitle"] = ", ".join([x.nazwa for x in wydzialy])
        return context

    def get_table_kwargs(self):
        uczelnia = Uczelnia.objects.all().first()
        pokazuj = uczelnia.pokazuj_liczbe_cytowan_w_rankingu

        if pokazuj == OpcjaWyswietlaniaField.POKAZUJ_NIGDY or (
            pokazuj == OpcjaWyswietlaniaField.POKAZUJ_ZALOGOWANYM
            and self.request.user.is_anonymous
        ):
            return {"exclude": ("liczba_cytowan_sum",)}
        return {}



from django.urls import path

from import_list_if.views import (
    ImportListIfDetailsView,
    ImportListIfResultsView,
    ImportListIfRouterView,
    ListaImportowView,
    NowyImportView,
    RestartImportView,
)

app_name = "import_list_if"

urlpatterns = [
    path("", ListaImportowView.as_view(), name="index"),
    path("new/", NowyImportView.as_view(), name="new"),
    path(
        "<uuid:pk>/",
        ImportListIfRouterView.as_view(),
        name="importlistif-router",
    ),
    path(
        "<uuid:pk>/detale/",
        ImportListIfDetailsView.as_view(),
        name="importlistif-details",
    ),
    path(
        "<uuid:pk>/rezultaty/",
        ImportListIfResultsView.as_view(),
        name="importlistif-results",
    ),
    path(
        "<uuid:pk>/regen/",
        RestartImportView.as_view(),
        name="restart",
    ),
]



# Generated by Django 3.2.15 on 2022-10-02 21:38

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("dynamic_columns", "0002_auto_20221002_2252"),
    ]

    operations = [
        migrations.AddField(
            model_name="modeladmin",
            name="model_ref",
            field=models.ForeignKey(
                default=1,
                on_delete=django.db.models.deletion.CASCADE,
                to="contenttypes.contenttype",
            ),
            preserve_default=False,
        ),
    ]



"""W tym pakiecie znajdują się procedury generujące raporty, które są dostępne
"od ręki" -- generowane za pomocą WWW"""
from django.http import Http404
from django.http.response import HttpResponseRedirect
from django.urls import reverse
from django.views.generic import DetailView, View
from django.views.generic.base import TemplateView
from django.views.generic.edit import BaseDeleteView, FormView
from sendfile import sendfile

from celeryui.interfaces import IWebTask
from celeryui.models import Report
from .ranking_autorow import *  # noqa
from .raport_jednostek_2012 import *  # noqa

from django.contrib import messages

from bpp.models import Rekord
from bpp.views.raporty.forms import (
    RankingAutorowForm,
    RaportAutorowForm,
    RaportDlaKomisjiCentralnejForm,
    RaportJednostekForm,
)


class PobranieRaportu(View):
    """Domyślnie dajemy wszystkim zalogowanym, którzy znają link,
    możliwość pobrania raportu."""

    def get(self, request, uid, *args, **kwargs):
        try:
            raport = Report.objects.filter(uid=uid).exclude(finished_on=None)[0]
        except IndexError:
            raise Http404

        return sendfile(request, raport.file.path, attachment=True)


class PodgladRaportu(DetailView):
    """Domyślnie dajemy wszystkim zalogowanym, którzy znają link,
    możliwość podejrzenia raportu."""

    template_name = "raporty/podglad_raportu.html"

    def get_object(self, queryset=None):
        try:
            raport = Report.objects.get(uid=self.kwargs["uid"])
        except Report.DoesNotExist:
            raise Http404

        return raport


class KasowanieRaportu(BaseDeleteView):
    slug_field = "uid"
    success_url = "../../.."

    def get_object(self):
        try:
            return Report.objects.get(
                uid=self.kwargs["uid"], ordered_by=self.request.user
            )
        except Report.DoesNotExist:
            raise Http404

    def delete(self, request, *args, **kwargs):
        obj = self.get_object()

        try:
            obj.file.path
        except ValueError:
            pass

        response = BaseDeleteView.delete(self, request, *args, **kwargs)
        messages.add_message(request, messages.INFO, "Raport został usunięty.")
        return response

    def render_to_response(self, *args, **kwargs):
        return self.delete(self.request, *args, **kwargs)


class RaportyMixin:
    def get_raporty(self):
        raporty = Report.objects.filter(ordered_by=self.request.user)
        for raport in raporty:
            raport.adapted = IWebTask(raport)
        return raporty

    def get_lata(self):
        return (
            Rekord.objects.all()
            .values_list("rok", flat=True)
            .order_by("rok")
            .distinct()
        )


class RaportyFormMixin(RaportyMixin):
    template_name = "raporty/strona_raportow/podstrona.html"
    success_url = "."
    nazwa_raportu = "skonfiguruj-mnie"

    def get_context_data(self, **kwargs):
        data = FormView.get_context_data(self, **kwargs)
        data["raporty"] = []
        if not self.request.user.is_anonymous:
            data["raporty"] = self.get_raporty()
        data["nazwa_raportu"] = self.nazwa_raportu
        return data

    def get_form_kwargs(self, **kw):
        data = FormView.get_form_kwargs(self, **kw)
        data["lata"] = self.get_lata()
        return data

    def get_raport_arguments(self, form):
        return form.cleaned_data

    def form_valid(self, form):
        r = Report.objects.create(
            ordered_by=self.request.user,
            function=self.request.POST["report"],
            arguments=self.get_raport_arguments(form),
        )

        if self.request.POST.get("do_not_call_celery") != "1":
            from bpp.tasks import make_report

            make_report.delay(r.uid)

        messages.add_message(
            self.request,
            messages.INFO,
            "Zamówienie raportu zostało złożone. Zostaniesz poinformowany/a "
            "na bieżąco o zakończeniu jego generowania.",
        )

        return FormView.form_valid(self, form)


class RaportCommonView(RaportyFormMixin, FormView):
    def form_valid(self, form):
        rok_min = form.cleaned_data["od_roku"]
        rok_max = form.cleaned_data["do_roku"]
        object = form.cleaned_data[self.object_key].pk
        output = "html"
        if form.cleaned_data["output"]:
            output = "msw"

        if rok_min != rok_max:
            url = (
                reverse(self.url_min_max, args=(object, rok_min, rok_max))
                + "?output=%s" % output
            )
            return HttpResponseRedirect(url)

        url = reverse(self.url_single, args=(object, rok_min)) + "?output=%s" % output
        return HttpResponseRedirect(url)


class RaportJednostek(RaportCommonView):
    form_class = RaportJednostekForm
    nazwa_raportu = "Raport jednostek"
    object_key = "jednostka"
    url_min_max = "bpp:raport-jednostek-rok-min-max"
    url_single = "bpp:raport-jednostek"


class RaportAutorow(RaportCommonView):
    form_class = RaportAutorowForm
    nazwa_raportu = "Raport autorów"
    object_key = "autor"
    url_min_max = "bpp:raport-autorow-rok-min-max"
    url_single = "bpp:raport-autorow"
    #
    # def form_valid(self, form):
    #     rok_min = form.cleaned_data['od_roku']
    #     rok_max = form.cleaned_data['do_roku']
    #     autor = form.cleaned_data['autor'].pk
    #
    #     if rok_min != rok_max:
    #         return HttpResponseRedirect(
    #             reverse(
    #                 "bpp:raport-autorow-rok-min-max",
    #                 args=(autor, rok_min, rok_max)))
    #
    #     return HttpResponseRedirect(
    #         reverse("bpp:raport-autorow", args=(autor, rok_min)))


class RankingAutorowFormularz(RaportyFormMixin, FormView):
    form_class = RankingAutorowForm
    nazwa_raportu = "Ranking autorow"

    def form_valid(self, form):
        url = reverse(
            "bpp:ranking-autorow",
            args=(
                form.cleaned_data["od_roku"],
                form.cleaned_data["do_roku"],
            ),
        )

        w = form.cleaned_data["wydzialy"]
        if w:
            url += "?"
            url += "&".join(["wydzialy[]=%s" % x.pk for x in w])

        e = form.cleaned_data["_export"]
        if e:
            if url.find("?") < 0:
                url += "?"
            else:
                url += "&"
            url += f"_export={ e }"

        r = form.cleaned_data["rozbij_na_jednostki"]
        if url.find("?") < 0:
            url += "?"
        else:
            url += "&"
        url += f"rozbij_na_jednostki={ r }"

        ta = form.cleaned_data["tylko_afiliowane"]
        if url.find("?") < 0:
            url += "?"
        else:
            url += "&"
        url += f"tylko_afiliowane={ ta }"

        return HttpResponseRedirect(url)


class RaportDlaKomisjiCentralnejFormularz(RaportyFormMixin, FormView):
    form_class = RaportDlaKomisjiCentralnejForm
    nazwa_raportu = "Raport dla Komisji Centralnej"

    def get_raport_arguments(self, form):
        form.cleaned_data["autor"] = form.cleaned_data["autor"].pk
        return form.cleaned_data


class RaportSelector(RaportyMixin, TemplateView):
    template_name = "raporty/strona_raportow/index.html"

    def get_context_data(self, **kwargs):
        return super().get_context_data(raporty=self.get_raporty(), **kwargs)



from typing import Union

from django.db.models import Q, Value
from django.db.models.functions import Lower, Replace, Trim

from .normalization import (
    normalize_doi,
    normalize_funkcja_autora,
    normalize_grupa_pracownicza,
    normalize_isbn,
    normalize_kod_dyscypliny,
    normalize_nazwa_dyscypliny,
    normalize_nazwa_jednostki,
    normalize_nazwa_wydawcy,
    normalize_public_uri,
    normalize_tytul_naukowy,
    normalize_tytul_publikacji,
    normalize_tytul_zrodla,
    normalize_wymiar_etatu,
)

from django.contrib.contenttypes.models import ContentType
from django.contrib.postgres.search import TrigramSimilarity

from bpp.models import (
    Autor,
    Autor_Jednostka,
    Dyscyplina_Naukowa,
    Funkcja_Autora,
    Grupa_Pracownicza,
    Jednostka,
    Rekord,
    Tytul,
    Wydawca,
    Wydawnictwo_Ciagle,
    Wydawnictwo_Zwarte,
    Wydzial,
    Wymiar_Etatu,
    Zrodlo,
)
from bpp.util import fail_if_seq_scan


def matchuj_wydzial(nazwa):
    try:
        return Wydzial.objects.get(nazwa__iexact=nazwa.strip())
    except Wydzial.DoesNotExist:
        pass


def matchuj_tytul(tytul: str, create_if_not_exist=False) -> Tytul:
    """
    Dostaje tytuł: pełną nazwę albo skrót
    """

    try:
        return Tytul.objects.get(nazwa__iexact=tytul)
    except (Tytul.DoesNotExist, Tytul.MultipleObjectsReturned):
        return Tytul.objects.get(skrot=normalize_tytul_naukowy(tytul))


def matchuj_funkcja_autora(funkcja_autora: str) -> Funkcja_Autora:
    funkcja_autora = normalize_funkcja_autora(funkcja_autora)
    return Funkcja_Autora.objects.get(
        Q(nazwa__iexact=funkcja_autora) | Q(skrot__iexact=funkcja_autora)
    )


def matchuj_grupa_pracownicza(grupa_pracownicza: str) -> Grupa_Pracownicza:
    grupa_pracownicza = normalize_grupa_pracownicza(grupa_pracownicza)
    return Grupa_Pracownicza.objects.get(nazwa__iexact=grupa_pracownicza)


def matchuj_wymiar_etatu(wymiar_etatu: str) -> Wymiar_Etatu:
    wymiar_etatu = normalize_wymiar_etatu(wymiar_etatu)
    return Wymiar_Etatu.objects.get(nazwa__iexact=wymiar_etatu)


def matchuj_jednostke(nazwa, wydzial=None):
    nazwa = normalize_nazwa_jednostki(nazwa)

    try:
        return Jednostka.objects.get(Q(nazwa__iexact=nazwa) | Q(skrot__iexact=nazwa))
    except Jednostka.DoesNotExist:
        if nazwa.endswith("."):
            nazwa = nazwa[:-1].strip()

        try:
            return Jednostka.objects.get(
                Q(nazwa__istartswith=nazwa) | Q(skrot__istartswith=nazwa)
            )
        except Jednostka.MultipleObjectsReturned as e:
            if wydzial is None:
                raise e

        return Jednostka.objects.get(
            Q(nazwa__istartswith=nazwa) | Q(skrot__istartswith=nazwa),
            Q(wydzial__nazwa__iexact=wydzial),
        )

    except Jednostka.MultipleObjectsReturned as e:
        if wydzial is None:
            raise e

        return Jednostka.objects.get(
            Q(nazwa__iexact=nazwa) | Q(skrot__iexact=nazwa),
            Q(wydzial__nazwa__iexact=wydzial),
        )


def matchuj_autora(
    imiona: str,
    nazwisko: str,
    jednostka: Union[Jednostka, None] = None,
    bpp_id: Union[int, None] = None,
    pbn_uid_id: Union[str, None] = None,
    system_kadrowy_id: Union[int, None] = None,
    pbn_id: Union[int, None] = None,
    orcid: Union[str, None] = None,
    tytul_str: Union[Tytul, None] = None,
):
    if bpp_id is not None:
        try:
            return Autor.objects.get(pk=bpp_id)
        except Autor.DoesNotExist:
            pass

    if orcid:
        try:
            return Autor.objects.get(orcid__iexact=orcid.strip())
        except Autor.DoesNotExist:
            pass

    if pbn_uid_id is not None and pbn_uid_id.strip() != "":
        # Może być > 1 autor z takim pbn_uid_id
        _qset = Autor.objects.filter(pbn_uid_id=pbn_uid_id)
        if _qset.exists():
            return _qset.first()

    if system_kadrowy_id is not None:
        try:
            int(system_kadrowy_id)
        except (TypeError, ValueError):
            system_kadrowy_id = None

        if system_kadrowy_id is not None:
            try:
                return Autor.objects.get(system_kadrowy_id=system_kadrowy_id)
            except Autor.DoesNotExist:
                pass

    if pbn_id is not None:
        if isinstance(pbn_id, str):
            pbn_id = pbn_id.strip()

        try:
            pbn_id = int(pbn_id)
        except (TypeError, ValueError):
            pbn_id = None

        if pbn_id is not None:
            try:
                return Autor.objects.get(pbn_id=pbn_id)
            except Autor.DoesNotExist:
                pass

    queries = [
        Q(
            Q(nazwisko__iexact=nazwisko.strip())
            | Q(poprzednie_nazwiska__icontains=nazwisko.strip()),
            imiona__iexact=imiona.strip(),
        )
    ]

    if tytul_str:
        queries.append(queries[0] & Q(tytul__skrot=tytul_str))

    for qry in queries:
        try:
            return Autor.objects.get(qry)
        except (Autor.DoesNotExist, Autor.MultipleObjectsReturned):
            pass

        try:
            return Autor.objects.get(qry & Q(aktualna_jednostka=jednostka))
        except (Autor.MultipleObjectsReturned, Autor.DoesNotExist):
            pass

    # Jesteśmy tutaj. Najwyraźniej poszukiwanie po aktualnej jednostce, imieniu, nazwisku,
    # tytule itp nie bardzo się powiodło. Spróbujmy innej strategii -- jednostka jest
    # określona, poszukajmy w jej autorach. Wszak nie musi być ta jednostka jednostką
    # aktualną...

    if jednostka:

        queries = [
            Q(
                Q(autor__nazwisko__iexact=nazwisko.strip())
                | Q(autor__poprzednie_nazwiska__icontains=nazwisko.strip()),
                autor__imiona__iexact=imiona.strip(),
            )
        ]
        if tytul_str:
            queries.append(queries[0] & Q(autor__tytul__skrot=tytul_str))

        for qry in queries:
            try:
                return jednostka.autor_jednostka_set.get(qry).autor
            except (
                Autor_Jednostka.MultipleObjectsReturned,
                Autor_Jednostka.DoesNotExist,
            ):
                pass

    return None


def matchuj_zrodlo(
    s: Union[str, None],
    issn: Union[str, None] = None,
    e_issn: Union[str, None] = None,
    alt_nazwa=None,
) -> Union[None, Zrodlo]:
    if s is None or str(s) == "":
        return

    if issn is not None:
        try:
            return Zrodlo.objects.get(issn=issn)
        except (Zrodlo.DoesNotExist, Zrodlo.MultipleObjectsReturned):
            pass

    if e_issn is not None:
        try:
            return Zrodlo.objects.get(e_issn=e_issn)
        except (Zrodlo.DoesNotExist, Zrodlo.MultipleObjectsReturned):
            pass

    for elem in s, alt_nazwa:
        if elem is None:
            continue

        elem = normalize_tytul_zrodla(elem)
        try:
            return Zrodlo.objects.get(Q(nazwa__iexact=elem) | Q(skrot__iexact=elem))
        except Zrodlo.MultipleObjectsReturned:
            pass
        except Zrodlo.DoesNotExist:
            if elem.endswith("."):
                try:
                    return Zrodlo.objects.get(
                        Q(nazwa__istartswith=elem[:-1])
                        | Q(skrot__istartswith=elem[:-1])
                    )
                except Zrodlo.DoesNotExist:
                    pass
                except Zrodlo.MultipleObjectsReturned:
                    pass


def matchuj_dyscypline(kod, nazwa):
    nazwa = normalize_nazwa_dyscypliny(nazwa)
    try:
        return Dyscyplina_Naukowa.objects.get(nazwa=nazwa)
    except Dyscyplina_Naukowa.DoesNotExist:
        pass
    except Dyscyplina_Naukowa.MultipleObjectsReturned:
        pass

    kod = normalize_kod_dyscypliny(kod)
    try:
        return Dyscyplina_Naukowa.objects.get(kod=kod)
    except Dyscyplina_Naukowa.DoesNotExist:
        pass
    except Dyscyplina_Naukowa.MultipleObjectsReturned:
        pass


def matchuj_wydawce(nazwa, pbn_uid_id=None, similarity=0.9):
    nazwa = normalize_nazwa_wydawcy(nazwa)
    try:
        return Wydawca.objects.get(nazwa=nazwa, alias_dla_id=None)
    except Wydawca.DoesNotExist:
        pass

    if pbn_uid_id is not None:

        try:
            return Wydawca.objects.get(pbn_uid_id=pbn_uid_id)
        except Wydawca.DoesNotExist:
            pass

    loose = (
        Wydawca.objects.annotate(similarity=TrigramSimilarity("nazwa", nazwa))
        .filter(similarity__gte=similarity)
        .order_by("-similarity")[:5]
    )
    if loose.count() > 0 and loose.count() < 2:
        return loose.first()


TITLE_LIMIT_SINGLE_WORD = 15
TITLE_LIMIT_MANY_WORDS = 25

MATCH_SIMILARITY_THRESHOLD = 0.95
MATCH_SIMILARITY_THRESHOLD_LOW = 0.90
MATCH_SIMILARITY_THRESHOLD_VERY_LOW = 0.80

# Znormalizowany tytuł w bazie danych -- wyrzucony ciąg znaków [online], podwójne
# spacje pozamieniane na pojedyncze, trim całości
normalized_db_title = Trim(
    Replace(
        Replace(Lower("tytul_oryginalny"), Value(" [online]"), Value("")),
        Value("  "),
        Value(" "),
    )
)

# Znormalizowany skrót nazwy źródła -- wyrzucone spacje i kropki, trim, zmniejszone
# znaki
normalized_db_zrodlo_skrot = Trim(
    Replace(
        Replace(
            Replace(Lower("skrot"), Value(" "), Value("")),
            Value("-"),
            Value(""),
        ),
        Value("."),
        Value(""),
    )
)


def normalize_zrodlo_skrot_for_db_lookup(s):
    return s.lower().replace(" ", "").strip().replace("-", "").replace(".", "")


# Znormalizowany skrot zrodla do wyszukiwania -- wyrzucone wszystko procz kropek
normalized_db_zrodlo_nazwa = Trim(
    Replace(Lower("nazwa"), Value(" "), Value("")),
)


def normalize_zrodlo_nazwa_for_db_lookup(s):
    return s.lower().replace(" ", "").strip()


normalized_db_isbn = Trim(Replace(Lower("isbn"), Value("-"), Value("")))


def matchuj_publikacje(
    klass: [Wydawnictwo_Zwarte, Wydawnictwo_Ciagle, Rekord],
    title,
    year,
    doi=None,
    public_uri=None,
    isbn=None,
    zrodlo=None,
    DEBUG_MATCHOWANIE=False,
    isbn_matchuj_tylko_nadrzedne=True,
    doi_matchuj_tylko_nadrzedne=True,
):

    if doi is not None:
        doi = normalize_doi(doi)
        if doi:
            zapytanie = klass.objects.filter(doi__istartswith=doi, rok=year)

            if doi_matchuj_tylko_nadrzedne:
                if hasattr(klass, "wydawnictwo_nadrzedne_id"):
                    zapytanie = zapytanie.filter(wydawnictwo_nadrzedne_id=None)

            res = zapytanie.annotate(
                podobienstwo=TrigramSimilarity(normalized_db_title, title.lower())
            ).order_by("-podobienstwo")[:2]
            fail_if_seq_scan(res, DEBUG_MATCHOWANIE)
            if res.exists():
                if res.first().podobienstwo >= MATCH_SIMILARITY_THRESHOLD_VERY_LOW:
                    return res.first()

    title = normalize_tytul_publikacji(title)

    title_has_spaces = False

    if title is not None:
        title_has_spaces = title.find(" ") > 0

    if title is not None and (
        (not title_has_spaces and len(title) >= TITLE_LIMIT_SINGLE_WORD)
        or (title_has_spaces and len(title) >= TITLE_LIMIT_MANY_WORDS)
    ):
        if zrodlo is not None and hasattr(klass, "zrodlo"):
            try:
                return klass.objects.get(
                    tytul_oryginalny__istartswith=title, rok=year, zrodlo=zrodlo
                )
            except klass.DoesNotExist:
                pass
            except klass.MultipleObjectsReturned:
                print(
                    f"PPP ZZZ MultipleObjectsReturned dla title={title} rok={year} zrodlo={zrodlo}"
                )

    if (
        isbn is not None
        and isbn != ""
        and hasattr(klass, "isbn")
        and hasattr(klass, "e_isbn")
    ):
        ni = normalize_isbn(isbn)

        zapytanie = klass.objects.exclude(isbn=None, e_isbn=None).exclude(
            isbn="", e_isbn=""
        )

        if isbn_matchuj_tylko_nadrzedne:
            zapytanie = zapytanie.filter(wydawnictwo_nadrzedne_id=None)

            if klass == Rekord:
                zapytanie = zapytanie.filter(
                    pk__in=[
                        (ContentType.objects.get_for_model(Wydawnictwo_Zwarte).pk, x)
                        for x in Wydawnictwo_Zwarte.objects.wydawnictwa_nadrzedne_dla_innych()
                    ]
                )
            elif klass == Wydawnictwo_Zwarte:
                zapytanie = zapytanie.filter(
                    pk__in=Wydawnictwo_Zwarte.objects.wydawnictwa_nadrzedne_dla_innych()
                )
            else:
                raise NotImplementedError(
                    "Matchowanie po ISBN dla czegoś innego niż wydawnictwo zwarte nie opracowane"
                )

        #
        # Uwaga uwaga uwaga.
        #
        # Gdy matchujemy ISBN, to w BPP dochodzi do takiej nieciekawej sytuacji: wpisywany jest
        # ISBN zarówno dla rozdziałów jak i dla wydawnictw nadrzędnych.
        #
        # Zatem, na ten moment, aby usprawnić matchowanie ISBN, jeżeli ustawiona jest flaga
        # isbn_matchuj_tylko_nadrzedne, to system bedzie szukał tylko i wyłącznie wśród
        # rekordów będących wydawnictwami nadrzędnymi (czyli nie mającymi rekordów podrzędnych)
        #

        res = (
            zapytanie.filter(Q(isbn=ni) | Q(e_isbn=ni))
            .annotate(
                podobienstwo=TrigramSimilarity(
                    normalized_db_title,
                    title.lower(),
                )
            )
            .order_by("-podobienstwo")[:2]
        )
        fail_if_seq_scan(res, DEBUG_MATCHOWANIE)
        if res.exists():
            if res.first().podobienstwo >= MATCH_SIMILARITY_THRESHOLD_VERY_LOW:
                return res.first()

    public_uri = normalize_public_uri(public_uri)
    if public_uri:
        res = (
            klass.objects.filter(Q(www=public_uri) | Q(public_www=public_uri))
            .annotate(
                podobienstwo=TrigramSimilarity(normalized_db_title, title.lower())
            )
            .order_by("-podobienstwo")[:2]
        )
        fail_if_seq_scan(res, DEBUG_MATCHOWANIE)
        if res.exists():

            if res.first().podobienstwo >= MATCH_SIMILARITY_THRESHOLD:
                return res.first()

    if title is not None and (
        (not title_has_spaces and len(title) >= TITLE_LIMIT_SINGLE_WORD)
        or (title_has_spaces and len(title) >= TITLE_LIMIT_MANY_WORDS)
    ):
        res = (
            klass.objects.filter(tytul_oryginalny__istartswith=title, rok=year)
            .annotate(
                podobienstwo=TrigramSimilarity(normalized_db_title, title.lower())
            )
            .order_by("-podobienstwo")[:2]
        )

        fail_if_seq_scan(res, DEBUG_MATCHOWANIE)
        if res.exists():
            if res.first().podobienstwo >= MATCH_SIMILARITY_THRESHOLD:
                return res.first()

        # Ostatnia szansa, po podobieństwie, niski próg

        res = (
            klass.objects.filter(rok=year)
            .annotate(
                podobienstwo=TrigramSimilarity(normalized_db_title, title.lower())
            )
            .order_by("-podobienstwo")[:2]
        )

        fail_if_seq_scan(res, DEBUG_MATCHOWANIE)
        if res.exists():
            if res.first().podobienstwo >= MATCH_SIMILARITY_THRESHOLD_LOW:
                return res.first()



# Generated by Django 3.0.14 on 2021-10-11 08:26

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0305_merge_20211010_2323"),
    ]

    operations = [
        migrations.DeleteModel(
            name="Ewaluacja2021LiczbaNDlaUczelni",
        ),
    ]



# Generated by Django 2.1.7 on 2019-08-12 12:12

from django.db import migrations, models

from bpp import const


def zmigruj_charaktery(apps, schema_editor):
    Charakter_Formalny = apps.get_model("bpp", "Charakter_Formalny")
    db_alias = schema_editor.connection.alias
    for elem in Charakter_Formalny.objects.using(db_alias).all():
        needs_saving = False
        if elem.nazwa_w_primo == "Rozdział" or elem.rozdzial_pbn:
            elem.charakter_sloty = const.CHARAKTER_SLOTY_ROZDZIAL
            needs_saving = True

        if elem.nazwa_w_primo == "Książka" or elem.ksiazka_pbn:
            elem.charakter_sloty = const.CHARAKTER_SLOTY_KSIAZKA
            needs_saving = True

        if needs_saving:
            elem.save()


class Migration(migrations.Migration):
    dependencies = [
        ("bpp", "0172_auto_20190806_2052"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="wydawca",
            options={"verbose_name": "wydawca", "verbose_name_plural": "wydawcy"},
        ),
        migrations.AddField(
            model_name="charakter_formalny",
            name="charakter_sloty",
            field=models.PositiveSmallIntegerField(
                blank=True,
                choices=[(1, "Książka"), (2, "Rozdział")],
                default=None,
                help_text="Jak potraktować ten charakter przy kalkulacji slotów?",
                null=True,
                verbose_name="Charakter dla slotów",
            ),
        ),
        migrations.RunPython(zmigruj_charaktery, migrations.RunPython.noop),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-08-24 00:19
from __future__ import unicode_literals

from django.db import migrations

from bpp.migration_util import load_custom_sql


class Migration(migrations.Migration):
    dependencies = [
        ('bpp', '0104_nowe_sumy_view'),
    ]

    operations = [
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql("0105_rekord_z_konferencja")),

    ]



# -*- encoding: utf-8 -*-

try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse
from django.test import TestCase, Client
from bpp.models.profile import BppUser
from bpp.models.struktura import Wydzial
from bpp.models.system import Typ_Odpowiedzialnosci, Charakter_Formalny, Typ_KBN
from bpp.models.wydawnictwo_ciagle import Wydawnictwo_Ciagle_Autor, Wydawnictwo_Ciagle
from bpp.tests.tests_legacy.test_reports.util import stworz_obiekty_dla_raportow
from bpp.tests.util import (
    any_wydzial,
    any_autor,
    any_ciagle,
    any_jednostka,
    CURRENT_YEAR,
)
from bpp.views.raporty.ranking_autorow import RankingAutorow


class TestRankingAutorow(TestCase):

    # fixtures = ['typ_odpowiedzialnosci.json',
    #             'charakter_formalny.json',
    #             'typ_kbn.json']

    def setUp(self):
        stworz_obiekty_dla_raportow()

        w1 = any_wydzial(nazwa="Wydzial 1", skrot="W9")
        w1.zezwalaj_na_ranking_autorow = True
        w1.save()
        j1 = any_jednostka(wydzial=w1, uczelnia=w1.uczelnia)

        w2 = any_wydzial(nazwa="Wydzial 2", skrot="W8")
        w2.zezwalaj_na_ranking_autorow = True
        w2.save()
        j2 = any_jednostka(wydzial=w2, uczelnia=w2.uczelnia)

        self.w2 = w2

        a1 = any_autor()

        wejdzie1 = any_ciagle(impact_factor=33.333)
        Wydawnictwo_Ciagle_Autor.objects.create(
            rekord=wejdzie1,
            autor=a1,
            jednostka=j1,
            kolejnosc=1,
            typ_odpowiedzialnosci=Typ_Odpowiedzialnosci.objects.get(skrot="aut."),
        )

        wejdzie2 = any_ciagle(impact_factor=44.444)
        Wydawnictwo_Ciagle_Autor.objects.create(
            rekord=wejdzie2,
            autor=a1,
            jednostka=j2,
            kolejnosc=1,
            typ_odpowiedzialnosci=Typ_Odpowiedzialnosci.objects.get(skrot="aut."),
        )

        nie_wejdzie = any_ciagle(
            typ_kbn=Typ_KBN.objects.get(skrot="PW"), impact_factor=55.555
        )
        Wydawnictwo_Ciagle_Autor.objects.create(
            rekord=nie_wejdzie,
            autor=a1,
            jednostka=j1,
            kolejnosc=1,
            typ_odpowiedzialnosci=Typ_Odpowiedzialnosci.objects.get(skrot="aut."),
        )

        BppUser.objects.create_user(username="foo", email="foo@bar.pl", password="bar")

        self.client = Client()
        response = self.client.post(
            reverse("login_form"), {"username": "foo", "password": "bar"}, follow=True
        )
        self.assertEqual(response.status_code, 200)

    def test_bez_argumentow(self):
        "Zsumuje punktacje ze wszystkich prac, ze wszystkich wydziałów dla danego roku"
        response = self.client.get(
            reverse(
                "bpp:ranking-autorow", args=(str(CURRENT_YEAR), str(CURRENT_YEAR),)
            ),
            follow=True,
        )
        # wydział 2
        self.assertIn("44,444", response.rendered_content)
        # wydział 1
        self.assertIn("33,333", response.rendered_content)

    def test_z_wydzialem(self):
        "Zsumuje punktacje ze wszystkich prac, ze wszystkich wydziałów dla danego roku"
        response = self.client.get(
            reverse("bpp:ranking-autorow", args=(str(CURRENT_YEAR), str(CURRENT_YEAR),))
            + "?wydzialy[]="
            + str(self.w2.pk),
            follow=True,
        )
        # wydział 2
        self.assertIn("44,444", response.rendered_content)
        # wydział 1 - praca nie wejdzie do rankingu
        self.assertNotIn("33,333", response.rendered_content)

    def test_bez_rozbicia(self):
        "Zsumuje punktacje ze wszystkich prac bez rozbicia na jednostki"
        response = self.client.get(
            reverse("bpp:ranking-autorow", args=(str(CURRENT_YEAR), str(CURRENT_YEAR),))
            + "?rozbij_na_jednostki=False",
            follow=True,
        )
        # suma punktacji
        self.assertIn("77,777", response.rendered_content)

    def test_eksport_csv(self):
        "XLS"
        response = self.client.get(
            reverse("bpp:ranking-autorow", args=(str(CURRENT_YEAR), str(CURRENT_YEAR),))
            + "?_export=csv",
            follow=True,
        )
        # suma punktacji
        self.assertIn(b",44.444,", response.content)



# Generated by Django 3.0.11 on 2021-03-14 21:04

from django.db import migrations, models

import bpp.models.uczelnia


class Migration(migrations.Migration):

    dependencies = [
        ("raport_slotow", "0009_auto_20210308_0846"),
    ]

    operations = [
        migrations.AlterField(
            model_name="raportslotowuczelnia",
            name="do_roku",
            field=models.IntegerField(
                default=bpp.models.uczelnia.UczelniaManager.do_roku_default
            ),
        ),
    ]



# Generated by Django 3.0.14 on 2021-07-13 09:17

from django.db import migrations

from bpp.migration_util import load_custom_sql


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0278_autorzy_profil_orcid"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="wydawnictwo_ciagle",
            name="ostatnio_zmieniony_dla_pbn",
        ),
        migrations.RemoveField(
            model_name="wydawnictwo_zwarte",
            name="ostatnio_zmieniony_dla_pbn",
        ),
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql(
                "0279_usun_ost_akt_pbn", app_name="bpp"
            ),
        ),
        # Odbudowanie tabeli rekord_mat spowoduje skasowanie bpp_uczelnia_ewaluacja_view,
        # stąd potrzeba zaciągnąć ten plik z poprzednich migracji:
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql("0207_uczelnia_analiza_view")
        ),
    ]



import os
import sys
import uuid

from django.core.files.base import ContentFile

from django.contrib.auth.models import Group

try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse

from django.http import Http404
from django.test.utils import override_settings
from model_bakery import baker

from celeryui.models import Report

from django.utils import timezone

from bpp.models import Charakter_Formalny, Jezyk, Typ_KBN, Typ_Odpowiedzialnosci
from bpp.tests.tests_legacy.testutil import UserTestCase
from bpp.tests.util import any_autor, any_ciagle, any_jednostka
from bpp.util import rebuild_contenttypes
from bpp.views.raporty import KasowanieRaportu, PodgladRaportu, RaportSelector


class TestRaportSelector(UserTestCase):
    def test_raportselector(self):
        p = RaportSelector()
        p.request = self.factory.get("/")
        p.get_context_data()

    def test_raportselector_with_reports(self):
        for x, kiedy_ukonczono in enumerate([timezone.now(), None]):
            baker.make(Report, arguments={}, file=None, finished_on=kiedy_ukonczono)

        self.client.get(reverse("bpp:raporty"))


class RaportMixin:
    def zrob_raport(self):
        r = baker.make(
            Report, file=None, function="kronika-uczelni", arguments='{"rok":"2013"}'
        )
        return r


class TestPobranieRaportu(RaportMixin, UserTestCase):
    def setUp(self):
        UserTestCase.setUp(self)
        self.r = self.zrob_raport()
        error_class = OSError
        if sys.platform.startswith("win"):
            error_class = WindowsError

        try:
            os.unlink(os.path.join(settings.MEDIA_ROOT, "raport", "test_raport"))
        except error_class:
            pass
        self.r.file.save("test_raport", ContentFile("hej ho"))

    def test_pobranie_nginx(self):
        # Raport musi byc zakonczony, ineczej nie ma pobrania
        self.r.finished_on = timezone.now()
        self.r.save()

        with override_settings(SENDFILE_BACKEND="sendfile.backends.nginx"):
            url = reverse("bpp:pobranie-raportu", kwargs=dict(uid=self.r.uid))
            resp = self.client.get(url)
            self.assertEqual(resp.status_code, 200)
            self.assertIn("x-accel-redirect", resp.headers)


class TestPodgladRaportu(RaportMixin, UserTestCase):
    def setUp(self):
        UserTestCase.setUp(self)
        self.r = self.zrob_raport()

    def test_podgladraportu(self):
        p = PodgladRaportu()
        p.kwargs = {}
        p.kwargs["uid"] = self.r.uid

        self.assertEqual(p.get_object(), self.r)

        p.kwargs["uid"] = str(uuid.uuid4())
        self.assertRaises(Http404, p.get_object)


class KasowanieRaportuMixin:
    def setUp(self):
        self.r = self.zrob_raport()
        self.r.ordered_by = self.user
        self.r.save()


class TestKasowanieRaportu(KasowanieRaportuMixin, RaportMixin, UserTestCase):
    def setUp(self):
        UserTestCase.setUp(self)
        KasowanieRaportuMixin.setUp(self)

    def test_kasowanieraportu(self):
        k = KasowanieRaportu()
        k.kwargs = dict(uid=self.r.uid)

        class FakeRequest:
            user = self.user

        k.request = FakeRequest()

        k.request.user = None
        self.assertRaises(Http404, k.get_object)

        k.request.user = self.user
        self.assertEqual(k.get_object(), self.r)

    def test_kasowanieraportu_client(self):
        self.assertEqual(Report.objects.count(), 1)
        url = reverse("bpp:kasowanie-raportu", kwargs=dict(uid=self.r.uid))
        resp = self.client.get(url)
        self.assertRedirects(resp, reverse("bpp:raporty"))
        self.assertEqual(Report.objects.count(), 0)


from django.conf import settings


class TestWidokiRaportJednostek2012(UserTestCase):
    # fixtures = ['charakter_formalny.json',
    #             'jezyk.json',
    #             'typ_kbn.json',
    #             'typ_odpowiedzialnosci.json']

    def setUp(self):
        UserTestCase.setUp(self)
        self.j = any_jednostka()
        Typ_KBN.objects.get_or_create(skrot="PW", nazwa="Praca wieloośrodkowa")
        Jezyk.objects.get_or_create(skrot="pol.", nazwa="polski")
        Charakter_Formalny.objects.get_or_create(
            skrot="KSZ", nazwa="Książka w języku obcym"
        )
        Charakter_Formalny.objects.get_or_create(
            skrot="KSP", nazwa="Książka w języku polskim"
        )
        Charakter_Formalny.objects.get_or_create(skrot="KS", nazwa="Książka")
        Charakter_Formalny.objects.get_or_create(skrot="ROZ", nazwa="Rozdział książki")
        Group.objects.get_or_create(name="wprowadzanie danych")

    def test_jeden_rok(self):
        url = reverse("bpp:raport-jednostek-rok-min-max", args=(self.j.pk, 2010, 2013))
        res = self.client.get(url)

        self.assertContains(
            res, "Dane o publikacjach za okres 2010 - 2013", status_code=200
        )

    def test_zakres_lat(self):
        url = reverse("bpp:raport-jednostek", args=(self.j.pk, 2013))
        res = self.client.get(url)

        self.assertContains(res, "Dane o publikacjach za rok 2013", status_code=200)


class TestRankingAutorow(UserTestCase):
    def setUp(self):
        UserTestCase.setUp(self)

        rebuild_contenttypes()

        Typ_Odpowiedzialnosci.objects.get_or_create(skrot="aut.", nazwa="autor")
        Group.objects.get_or_create(name="wprowadzanie danych")

        j = any_jednostka()
        a = any_autor(nazwisko="Kowalski")
        c = any_ciagle(impact_factor=200, rok=2000)
        c.dodaj_autora(a, j)

    def test_renderowanie(self):
        url = reverse("bpp:ranking-autorow", args=(2000, 2000))
        res = self.client.get(url)
        self.assertContains(res, "Ranking autorów", status_code=200)
        self.assertContains(res, "Kowalski")

    def test_renderowanie_csv(self):
        url = reverse("bpp:ranking-autorow", args=(2000, 2000))
        res = self.client.get(url, data={"_export": "csv"})
        self.assertContains(res, '"Kowalski Jan Maria, dr",Jednostka')



# -*- coding: utf-8 -*-


from django.db import migrations, models
import datetime


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0058_auto_20161113_2332'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='wydzial',
            name='archiwalny',
        ),
        migrations.RemoveField(
            model_name='wydzial',
            name='wirtualny',
        ),
        migrations.AddField(
            model_name='wydzial',
            name='otwarcie',
            field=models.DateField(default=datetime.date(2016, 11, 14), verbose_name=b'Data otwarcia wydzia\xc5\x82u', auto_now_add=True),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='wydzial',
            name='zamkniecie',
            field=models.DateField(null=True, verbose_name=b'Data zamkni\xc4\x99cia wydzia\xc5\x82u', blank=True),
        ),
        migrations.AddField(
            model_name='wydzial',
            name='zarzadzaj_automatycznie',
            field=models.BooleanField(default=True, help_text=b"Wydzia\xc5\x82 ten b\xc4\x99dzie dowolnie modyfikowany przez procedury importujace dane z zewn\xc4\x99trznych\n        system\xc3\xb3w informatycznych. W przypadku, gdy pole ma ustawion\xc4\x85 warto\xc5\x9b\xc4\x87 na 'fa\xc5\x82sz', wydzia\xc5\x82 ten mo\xc5\xbce by\xc4\x87", verbose_name=b'Zarz\xc4\x85dzaj automatycznie'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-11-25 11:03
from __future__ import unicode_literals

from decimal import Decimal

from django.db import migrations, models

from bpp.migration_util import load_custom_sql


class Migration(migrations.Migration):
    dependencies = [
        ('bpp', '0150_auto_20181125_1202'),
    ]

    operations = [
        migrations.AddField(
            model_name='patent',
            name='punktacja_snip',
            field=models.DecimalField(db_index=True, decimal_places=3, default=Decimal('0.000'),
                                      help_text='CiteScore SNIP (Source Normalized Impact per Paper)', max_digits=6,
                                      verbose_name='Punktacja SNIP'),
        ),
        migrations.AddField(
            model_name='praca_doktorska',
            name='punktacja_snip',
            field=models.DecimalField(db_index=True, decimal_places=3, default=Decimal('0.000'),
                                      help_text='CiteScore SNIP (Source Normalized Impact per Paper)', max_digits=6,
                                      verbose_name='Punktacja SNIP'),
        ),
        migrations.AddField(
            model_name='praca_habilitacyjna',
            name='punktacja_snip',
            field=models.DecimalField(db_index=True, decimal_places=3, default=Decimal('0.000'),
                                      help_text='CiteScore SNIP (Source Normalized Impact per Paper)', max_digits=6,
                                      verbose_name='Punktacja SNIP'),
        ),
        migrations.AddField(
            model_name='punktacja_zrodla',
            name='punktacja_snip',
            field=models.DecimalField(db_index=True, decimal_places=3, default=Decimal('0.000'),
                                      help_text='CiteScore SNIP (Source Normalized Impact per Paper)', max_digits=6,
                                      verbose_name='Punktacja SNIP'),
        ),
        migrations.AddField(
            model_name='uczelnia',
            name='pokazuj_punktacja_snip',
            field=models.BooleanField(default=True, verbose_name='Pokazuj punktację SNIP na stronie rekordu'),
        ),
        migrations.AddField(
            model_name='wydawnictwo_ciagle',
            name='punktacja_snip',
            field=models.DecimalField(db_index=True, decimal_places=3, default=Decimal('0.000'),
                                      help_text='CiteScore SNIP (Source Normalized Impact per Paper)', max_digits=6,
                                      verbose_name='Punktacja SNIP'),
        ),
        migrations.AddField(
            model_name='wydawnictwo_zwarte',
            name='punktacja_snip',
            field=models.DecimalField(db_index=True, decimal_places=3, default=Decimal('0.000'),
                                      help_text='CiteScore SNIP (Source Normalized Impact per Paper)', max_digits=6,
                                      verbose_name='Punktacja SNIP'),
        ),
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql("0151_snip")
        ),

    ]



# -*- encoding: utf-8 -*-
from django.core.management import call_command
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    """Tworzy testową bazę danych (polecenie do użytku przez dewelopera)"""

    def handle(self, *args, **options):
        call_command("test", "create_test_db.tests", "--keepdb", "-v 2")


# -*- coding: utf-8 -*-


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0052_skupia_pracownikow_zarzadzaj_automatycznie'),
    ]

    operations = [
        migrations.AddField(
            model_name='jednostka',
            name='uczelnia',
            field=models.ForeignKey(on_delete=models.CASCADE, default=1, to='bpp.Uczelnia'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='uczelnia',
            name='obca_jednostka',
            field=models.ForeignKey(on_delete=models.CASCADE, related_name='obca_jednostka', blank=True, to='bpp.Jednostka', help_text=b'\n    Jednostka skupiaj\xc4\x85ca autor\xc3\xb3w nieindeksowanych, nie b\xc4\x99d\xc4\x85cych pracownikami uczelni. Procedury importuj\xc4\x85ce\n    dane z zewn\xc4\x99trznych system\xc3\xb3w informatycznych b\xc4\x99d\xc4\x85 przypisywa\xc4\x87 do tej jednostki osoby, kt\xc3\xb3re zako\xc5\x84czy\xc5\x82y\n    prac\xc4\x99 na uczelni. ', null=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-06-21 22:57


from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0074_django110'),
    ]

    operations = [
        migrations.AlterField(
            model_name='patent',
            name='status_korekty',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bpp.Status_Korekty'),
        ),
        migrations.AlterField(
            model_name='praca_doktorska',
            name='status_korekty',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bpp.Status_Korekty'),
        ),
        migrations.AlterField(
            model_name='praca_habilitacyjna',
            name='status_korekty',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bpp.Status_Korekty'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle',
            name='status_korekty',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bpp.Status_Korekty'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='status_korekty',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bpp.Status_Korekty'),
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0015_auto_20150617_2247'),
    ]

    operations = [
        migrations.AddField(
            model_name='autor',
            name='pbn_id',
            field=models.IntegerField(help_text=b'Identyfikator w systemie Polskiej Bibliografii Naukowej (PBN)', null=True, verbose_name=b'Identyfikator PBN', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='jednostka',
            name='pbn_id',
            field=models.IntegerField(help_text=b'Identyfikator w systemie Polskiej Bibliografii Naukowej (PBN)', null=True, verbose_name=b'Identyfikator PBN', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='uczelnia',
            name='pbn_id',
            field=models.IntegerField(help_text=b'Identyfikator w systemie Polskiej Bibliografii Naukowej (PBN)', null=True, verbose_name=b'Identyfikator PBN', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='wydzial',
            name='pbn_id',
            field=models.IntegerField(help_text=b'Identyfikator w systemie Polskiej Bibliografii Naukowej (PBN)', null=True, verbose_name=b'Identyfikator PBN', blank=True),
            preserve_default=True,
        ),
    ]



from formdefaults.models import FormRepresentationManager
from formdefaults.util import full_name, get_python_class_by_name


def test_full_name():
    assert (
        full_name(FormRepresentationManager())
        == "formdefaults.models.FormRepresentationManager"
    )


def test_get_python_class_by_name():
    fn = "formdefaults.models.FormRepresentationManager"
    assert get_python_class_by_name(fn) == FormRepresentationManager



# Generated by Django 3.2.14 on 2022-08-01 18:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("zglos_publikacje", "0002_auto_20220710_2331"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="zgloszenie_publikacji",
            options={
                "ordering": ("-utworzono", "tytul_oryginalny"),
                "verbose_name": "zgłoszenie publikacji",
                "verbose_name_plural": "zgłoszenia publikacji",
            },
        ),
        migrations.AddField(
            model_name="zgloszenie_publikacji",
            name="rok",
            field=models.IntegerField(
                db_index=True,
                default=2022,
                help_text="Rok uwzględniany przy wyszukiwaniu i raportach\n        KBN/MNiSW)",
            ),
            preserve_default=False,
        ),
    ]



from django import forms

from bpp.forms.fields import ORCIDField
from bpp.models import Autor, Rekord


class UstawORCIDAutoraForm(forms.Form):
    autor = forms.ModelChoiceField(queryset=Autor.objects.all())

    orcid = ORCIDField()


class UstawStronyRekorduForm(forms.Form):
    rekord = forms.ModelChoiceField(queryset=Rekord.objects.all())
    strony = forms.CharField(max_length=512)


class UstawTomRekorduForm(forms.Form):
    rekord = forms.ModelChoiceField(queryset=Rekord.objects.all())
    tom = forms.CharField(max_length=512)


class UstawNrZeszytuRekorduForm(forms.Form):
    rekord = forms.ModelChoiceField(queryset=Rekord.objects.all())
    nr_zeszytu = forms.CharField(max_length=512)


class UstawStreszczenieRekorduForm(forms.Form):
    rekord = forms.ModelChoiceField(queryset=Rekord.objects.all())
    streszczenie = forms.CharField(max_length=10240)



# Generated by Django 2.2.10 on 2020-03-15 21:12

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0205_merge_20200315_2208"),
    ]

    operations = [
        migrations.CreateModel(
            name="Cache_Punktacja_Autora_Query_View",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("pkdaut", models.DecimalField(decimal_places=4, max_digits=20)),
                ("slot", models.DecimalField(decimal_places=4, max_digits=20)),
                (
                    "zapisani_autorzy_z_dyscypliny",
                    django.contrib.postgres.fields.ArrayField(
                        base_field=models.TextField(), size=None
                    ),
                ),
            ],
            options={
                "db_table": "bpp_cache_punktacja_autora_view",
                "ordering": ("autor__nazwisko", "dyscyplina__nazwa"),
                "managed": False,
            },
        ),
        migrations.AlterField(
            model_name="autor",
            name="aktualny",
            field=models.BooleanField(
                db_index=True,
                default=False,
                help_text="Jeżeli zaznaczone, pole to oznacza,\n    że autor jest aktualnie - na dziś dzień - przypisany do jakiejś jednostki w bazie danych i jego przypisanie\n    do tej jednostki nie zostało zakończone wraz z konkretną datą w\n    przeszłości.",
                verbose_name="Aktualny?",
            ),
        ),
    ]



# -*- encoding: utf-8 -*-
import os

import xlrd
from bpp.models import Funkcja_Autora, Jednostka, Autor, Autor_Jednostka, Wydzial, Opi_2012_Afiliacja_Do_Wydzialu

ignored_seen, wiecej_niz_jeden, brak_takiego, unseen_jed = set(), set(), set(), set()

def znajdz_lub_zrob_stanowisko(nf):
    try:
        return Funkcja_Autora.objects.get(nazwa=nf)
    except Funkcja_Autora.DoesNotExist:
        try:
            return Funkcja_Autora.objects.create(nazwa=nf, skrot=nf)
        except Exception as e:
            print(e, nf)


def dopasuj_jednostke(nazwa_jednostki, text_mangle):

    nazwa_jednostki = nazwa_jednostki.strip().replace("  ", " ")

    if nazwa_jednostki in text_mangle.zamiany_nazw_jednostek:
        nazwa_jednostki = text_mangle.zamiany_nazw_jednostek[nazwa_jednostki]

    if nazwa_jednostki.endswith('.'):
        nazwa_jednostki = nazwa_jednostki[:-1]

    kw = {'nazwa__icontains': nazwa_jednostki}

    if nazwa_jednostki in text_mangle.single_fitters:
        kw = {'nazwa': nazwa_jednostki}

    if nazwa_jednostki in text_mangle.ignorowane_jednostki:
        if nazwa_jednostki not in ignored_seen:
            ignored_seen.add(nazwa_jednostki)
            # print "... TA JEDNOSTKA ZOSTAJE IGNOROWANA W CALOSCI: ", nazwa_jednostki.encode('utf-8')
        return None

    try:
        return Jednostka.objects.get(**kw)
    except Jednostka.DoesNotExist:
        if nazwa_jednostki not in unseen_jed:
            print("=== BRAK JEDNOSTKI: ", nazwa_jednostki.encode('utf-8'))
            unseen_jed.add(nazwa_jednostki)


def dopasuj_autora(imiona, nazwisko, jednostka, stanowisko):
    a = Autor.objects.filter(imiona__icontains=imiona, nazwisko__icontains=nazwisko)
    cnt = a.count()

    if cnt == 0:
        if (imiona, nazwisko) not in brak_takiego:
            print("--- BRAK TAKIEGO AUTORA W BPP: ", imiona.encode('utf-8'), nazwisko.encode('utf-8'), "Stanowisko:", stanowisko)
            brak_takiego.add((imiona, nazwisko))
        return

    elif cnt > 1:
        # jeżeli jest więcej, niż jeden autor pod takimi danymi, spróbujemy dopasować
        # do niego również jego JEDNOSTKĘ, być może w ten sposób uda nam się wyróżnić
        # unikalnego autora?

        aj = list(Autor_Jednostka.objects.filter(
            autor__imiona__icontains=imiona,
            autor__nazwisko__icontains=nazwisko,
            jednostka__nazwa=jednostka).values('autor').distinct())

        if len(aj) == 1:
            return Autor.objects.get(pk=aj[0]['autor'])

        if (imiona, nazwisko) not in wiecej_niz_jeden:
            print("*** WIĘCEJ NIŻ JEDEN AUTOR W BPP: ", imiona.encode('utf-8'), nazwisko.encode('utf-8'), " MIMO DOPASOWANIA Z JENOSTKĄ!")
            wiecej_niz_jeden.add((imiona, nazwisko))
        return

    else:
        return list(a)[0]



def importuj_wiersz(imiona, nazwisko, nazwa_jednostki, stanowisko, rok, text_mangle):
    # Stanowisko
    funkcja = znajdz_lub_zrob_stanowisko(stanowisko)

    # Poszukujemy jednostki
    jednostka = dopasuj_jednostke(nazwa_jednostki, text_mangle)
    if jednostka is None:
        return

    # Poszukajmy autora:
    autor = dopasuj_autora(imiona, nazwisko, jednostka, funkcja.nazwa)
    if autor is None:
        return

    autor.dodaj_jednostke(jednostka, rok, funkcja)


def importuj_sheet_roczny(sheet, rok, text_mangle):

    labels = [x.value for x in sheet.row(0)]

    for a in range(len(labels)):
        if labels[a] == '':
            labels[a] = str(a)

    for nrow in range(1, sheet.nrows):
        row = sheet.row(nrow)
        dct = dict(list(zip(labels, row)))

        importuj_wiersz(
            dct['PRC IMIE'].value, dct['PRC NAZWISKO'].value,
            dct['OB_NAZWA'].value, dct['Stanowisko'].value,
            rok, text_mangle)


def mangle_labels(labels):
    """Pierwsze 'stanowisko' zostawiamy, następne
    przemianowujemy na stanowisko_2009, potem stanowisko_2010 i tak dalej,
    stopień i wydział również. """
    juz = False
    new_labels = []
    appender = 2008

    fun = lambda element: element
    new_fun = lambda element: element.strip() + "_" + str(appender)

    for element in labels:
        if element.lower() == 'stanowisko':
            if juz:
                fun = new_fun
                appender += 1
            juz = True

        new_labels.append(fun(element))
    return new_labels


def importuj_sheet_osoby_nie_ujete(sheet, text_mangle):
    labels = mangle_labels([x.value for x in sheet.row(2)])

    for nrow in range(3, sheet.nrows):
        row = sheet.row(nrow)
        dct = dict(list(zip(labels, row)))

        # Stanowisko
        funkcja = znajdz_lub_zrob_stanowisko(dct['Stanowisko'].value)

        # Poszukujemy jednostki
        jednostka = dopasuj_jednostke(dct['OB_NAZWA'].value, text_mangle)
        if jednostka is None:
            continue

        # Poszukajmy autora:
        autor = dopasuj_autora(dct['PRC IMIE'].value, dct['PRC NAZWISKO'].value, jednostka, funkcja.nazwa)
        if autor is None:
            continue

        for rok in range(2009, 2013):
            stanowisko = 'stanowisko_%s' % rok
            wydzial = 'Wydział_%s' % rok
            # jest jeszcze stopień

            if dct[stanowisko].value == '#N/D!':
                continue

            wn = dct[wydzial].value
            try:
                wydzial_rok = Wydzial.objects.get(nazwa=wn)
            except Wydzial.DoesNotExist:
                print("BRAK TAKIEGO WYDZIALU:", [
                    str(x).encode('utf-8') for x in (wn, autor, jednostka, rok)])
                continue

            Opi_2012_Afiliacja_Do_Wydzialu.objects.get_or_create(
                autor=autor, wydzial=wydzial_rok, rok=rok)

        autor.dodaj_jednostke(jednostka, rok, funkcja)



def importuj_afiliacje(plik_xls, text_mangle):
    """
    Importuje afiliacje z pliku XLS.

    :param plik_xls: nazwa pliku XLS (pełna ścieżka)
    :param text_mangle: klasa z atrybutami: ignorowane_jednostki,
    single_filters, zamiany_nazw_jednostek
    """
    book = xlrd.open_workbook(plik_xls)

    arkusze_ignorowane = ['razem', '2008']

    for name in book.sheet_names():

        sheet = book.sheet_by_name(name)

        if name in ['2009', '2010', '2011', '2012']:
            importuj_sheet_roczny(sheet, int(name), text_mangle)

        elif name.startswith("osoby"):
            importuj_sheet_osoby_nie_ujete(sheet, text_mangle)

        elif name in arkusze_ignorowane:
            pass

        else:
            raise Exception("Nieznany tytuł arkusza: %r" % name)


def importuj_imiona_sheet(sheet, wydzial):
    labels = ['_id', 'tytul', 'imiona', 'nazwisko', 'afiliacja']

    def zmien_imiona(a, dct):
        i = dct['imiona'].value
        if len(a.imiona) < len(i):
            a.imiona = i
            a.save()

    def poinformuj(ile, dct):
        print("*** DLA AUTORA %s %s JEST %s DOPASOWAN!!!" % tuple([
            str(x).encode('utf-8') for x in [dct['imiona'].value, dct['nazwisko'].value, ile]]))

    for nrow in range(0, sheet.nrows):
        row = sheet.row(nrow)
        dct = dict(list(zip(labels, row)))

        i = dct['imiona'].value.split(" ", 1)[0]

        autor = Autor.objects.filter(imiona__istartswith=i, nazwisko=dct['nazwisko'].value)
        cnt = autor.count()

        if cnt == 1:
            a = autor[0]
            zmien_imiona(a, dct)
            continue

        elif cnt == 0:
            poinformuj(0, dct)

        else:
            # 2 lub więcej -- sprawdź, ilu pasuje pod wydział podany jako parametr
            results = []
            for a in autor:
                if a.afiliacja_na_rok(2012, wydzial, rozszerzona=True):
                    results.append(a)

            if len(results)==1:
                zmien_imiona(results[0], dct)
                continue

            poinformuj(len(results), dct)







def importuj_imiona(plik_xls):
    book = xlrd.open_workbook(plik_xls)
    sheet = book.sheet_by_index(0)
    wydzial = Wydzial.objects.get(
        skrot=os.path.basename(plik_xls).split("-")[0])
    importuj_imiona_sheet(sheet, wydzial)


from asgiref.sync import sync_to_async

# https://stackoverflow.com/questions/71952949/serving-file-asyncronously-with-django-and-uvicorn

# https://github.com/django/channels/issues/1722


def patch_sync_to_async(*args, **kwargs):
    """
    Monkey Patch the sync_to_async decorator
    ---------------------------------------
    ASGIRef made a change in their defaults that has caused major problems
    for channels. The decorator below needs to be updated to use
    thread_sensitive=False, thats why we are patching it on our side for now.
    https://github.com/django/channels/blob/main/channels/http.py#L220
    """
    kwargs["thread_sensitive"] = False
    return sync_to_async(*args, **kwargs)



"""Klasy pomocnicze dla klas ModelAdmin które chcą korzystać z parametru
GET ``numer_zgloszenia`` czyli wypełniać dane wstępne wg zawartości zgłoszenia
o danym numerze ID. """
import math

from crossref_bpp.core import Komparator
from crossref_bpp.models import CrossrefAPICache
from import_common.normalization import normalize_title
from .util import CustomizableFormsetParamsAdminMixinWyrzucWDjango40

from bpp import const


class KorzystaZCrossRefAPIMixin:
    """Mixin dla klas korzystajacych z parametru 'identyfikator_doi', który to identyfikator
    powoduje wypełnienie wstępnych parametrów wg danych z CrossRef API
    """

    def get_crossref_api_data(self, request):
        if request.method == "GET":
            nz = request.GET.get(const.CROSSREF_API_PARAM, "")
            if nz:
                return CrossrefAPICache.objects.get_by_doi(nz)


class InlineMixin:
    def get_extra_json_list_length(self, json_data):
        return len(json_data.get(self.crossref_api_json_attrname, []))

    def get_extra(self, request, obj=None, **kwargs):
        z = self.get_crossref_api_data(request)
        if z is not None:
            return self.get_extra_json_list_length(z)

        return super().get_extra(request=request, obj=obj, **kwargs)


class KorzystaZCrossRefAPIAutorInlineMixin(InlineMixin, KorzystaZCrossRefAPIMixin):
    """Mixin dla klas Inline znajdujacych się w formualrzach ModelAdmin,
    korzystajacych ze wstepnie uzupełnianych danych przez CrossRef API.

    Funkcja ``get_extra`` zwraca ilosc formularzy, jaka powinna byc wyswietlona
    aby pokryc zapotrzebowanie na inicjalne wartosci dla autorów."""

    crossref_api_json_attrname = "author"


class KorzystaZCrossRefAPIStreszczenieInlineMixin(
    InlineMixin, KorzystaZCrossRefAPIMixin
):
    """Mixin dla klas Inline znajdujacych się w formualrzach ModelAdmin,
    korzystajacych ze wstepnie uzupełnianych danych przez CrossRef API.

    Funkcja ``get_extra`` zwraca ilosc formularzy, jaka powinna byc wyswietlona
    aby pokryc zapotrzebowanie na inicjalne wartosci dla autorów."""

    crossref_api_json_attrname = "abstract"

    def get_extra_json_list_length(self, json_data):
        if json_data.get(self.crossref_api_json_attrname) is not None:
            return 1
        return self.extra


class UzupelniajWstepneDanePoCrossRefAPIMixin(
    KorzystaZCrossRefAPIMixin, CustomizableFormsetParamsAdminMixinWyrzucWDjango40
):
    """Mixin dla klas ModelAdmin które chcą miec wypełniane parametry wg zgłoszenia
    z parametru requestu GET ``identyfikator_doi``.

    Obecnie dla Wydawnictwo_Ciagle.

    """

    def get_changeform_initial_data(self, request):
        z = self.get_crossref_api_data(request)
        if z is not None:
            title = z.get("title")
            if isinstance(title, list):
                title = ". ".join(title)
            title = normalize_title(title)

            zrodlo = Komparator.porownaj_container_title(
                z.get("container-title")[0]
            ).rekord_po_stronie_bpp
            if zrodlo is not None:
                zrodlo = zrodlo.pk

            wydawca_txt = ""
            wydawca_idx = Komparator.porownaj_publisher(
                z.get("publisher")
            ).rekord_po_stronie_bpp
            if wydawca_idx is None:
                wydawca_txt = z.get("publisher")

            charakter_formalny_pk = None
            charakter_formalny = Komparator.porownaj_type(
                z.get("type")
            ).rekord_po_stronie_bpp
            if charakter_formalny is not None:
                charakter_formalny_pk = charakter_formalny.pk

            jezyk_pk = None
            jezyk = Komparator.porownaj_language(
                z.get("language")
            ).rekord_po_stronie_bpp
            if jezyk is not None:
                jezyk_pk = jezyk.pk

            e_issn = None
            if z.get("issn-type", []):
                for _issn in z.get("issn-type"):
                    if _issn.get("type") == "electronic":
                        e_issn = _issn.get("value")
                        break

            licencja_pk = None
            licencja_ilosc_miesiecy = None
            for _licencja in z.get("license", []):
                licencja = Komparator.porownaj_license(_licencja).rekord_po_stronie_bpp
                if licencja is not None:
                    licencja_pk = licencja.pk
                    try:
                        licencja_ilosc_miesiecy = int(
                            math.ceil(int(_licencja.get("delay-in-days")) / 30)
                        )
                    except (TypeError, ValueError):
                        pass
                    break

            ret = {
                "tytul_oryginalny": title,
                "zrodlo": zrodlo,
                "nr_zeszytu": z.get("issue", ""),
                "strony": z.get("page", ""),
                "slowa_kluczowe": ", ".join('"%s"' % x for x in z.get("subject", [])),
                "wydawca": wydawca_idx,
                "wydawca_opis": wydawca_txt,
                "doi": z.get("DOI"),
                "charakter_formalny": charakter_formalny_pk,
                "jezyk": jezyk_pk,
                "adnotacje": "Dodano na podstawie CrossRef API",
                "e_issn": e_issn,
                "openaccess_licencja": licencja_pk,
                "openaccess_ilosc_miesiecy": licencja_ilosc_miesiecy,
                "issn": z.get(
                    "ISSN",
                    [
                        None,
                    ],
                )[0],
                "www": z.get("resource", {}).get("primary", {}).get("URL", ""),
                "tom": z.get("volume", ""),
                "rok": z.get("published", {}).get("date-parts", [[""]])[0][0],
            }

            return ret

        return super().get_changeform_initial_data(request)

    def get_formset_kwargs(self, request, obj, inline, prefix):
        initial = []

        z = self.get_crossref_api_data(request)

        if z is None:
            return super().get_formset_kwargs(request, obj, inline, prefix)

        if (
            prefix == "autorzy_set"
            and obj.pk is None
            and str(type(inline)).find("<locals>.baseModel_AutorInline") > -1
        ):
            for zpa in z.get("author", []):
                rekord = Komparator.porownaj_author(zpa).rekord_po_stronie_bpp
                if rekord:
                    initial.append(
                        {
                            "autor": rekord,
                            # "jednostka": zpa.jednostka_id,
                            # "dyscyplina_naukowa": zpa.dyscyplina_naukowa_id,
                        }
                    )
                else:
                    initial.append({})

        elif prefix == "streszczenia" and obj.pk is None:
            if z.get("abstract") is not None:

                jezyk_pk = None
                jezyk = Komparator.porownaj_language(
                    z.get("language")
                ).rekord_po_stronie_bpp
                if jezyk is not None:
                    jezyk_pk = jezyk.pk

                initial.append(
                    {
                        "jezyk_streszczenia": jezyk_pk,
                        "streszczenie": z.get("abstract", "").replace("&#x0D;", "\n"),
                    }
                )

        return {
            "initial": initial,
        }



# -*- coding: utf-8 -*-


from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0003_auto_20141023_1419'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='patent_autor',
            unique_together=set([('rekord', 'autor', 'typ_odpowiedzialnosci'), ('rekord', 'autor', 'kolejnosc')]),
        ),
        migrations.AlterUniqueTogether(
            name='wydawnictwo_ciagle_autor',
            unique_together=set([('rekord', 'autor', 'typ_odpowiedzialnosci'), ('rekord', 'autor', 'kolejnosc')]),
        ),
        migrations.AlterUniqueTogether(
            name='wydawnictwo_zwarte_autor',
            unique_together=set([('rekord', 'autor', 'typ_odpowiedzialnosci'), ('rekord', 'autor', 'kolejnosc')]),
        ),
    ]



from django.db.models import Sum

from .ewaluacja3n_base import Ewaluacja3NBase
from .util import policz_knapsack

from bpp.models import Cache_Punktacja_Autora_Query
from bpp.util import pbar


class Plecakowy(Ewaluacja3NBase):
    def __init__(
        self,
        nazwa_dyscypliny="nauki medyczne",
        bez_limitu_uczelni=False,
        output_path=None,
    ):
        Ewaluacja3NBase.__init__(
            self=self, nazwa_dyscypliny=nazwa_dyscypliny, output_path=output_path
        )
        self.bez_limitu_uczelni = bez_limitu_uczelni
        self.get_data()

    @property
    def lista_prac(self):
        return self.lista_prac_tuples

    def get_lista_autorow_w_kolejnosci(self):

        punkty_na_id_autora = {
            i["autor_id"]: i["pkdaut__sum"]
            for i in Cache_Punktacja_Autora_Query.objects.values("autor_id").annotate(
                Sum("pkdaut")
            )
        }

        id_autorow = sorted(
            list(self.id_wszystkich_autorow),
            key=lambda item: punkty_na_id_autora[item],
            reverse=True,
        )

        return id_autorow

    def sumuj(self):

        for autor_id in pbar(self.get_lista_autorow_w_kolejnosci()):

            if self.bez_limitu_uczelni:
                wszystkie = list(
                    praca for praca in self.lista_prac_db.filter(autor_id=autor_id)
                )
            else:
                wszystkie = list(
                    praca
                    for praca in self.lista_prac_db.filter(autor_id=autor_id)
                    if self.czy_moze_przejsc_warunek_uczelnia(praca)
                )

            monografie = [x for x in wszystkie if x.monografia]
            nie_monografie = [x for x in wszystkie if not x.monografia]

            maksymalny_slot_za_calosc = self.maks_pkt_aut_calosc.get(autor_id)
            maksymalny_slot_za_monografie = self.maks_pkt_aut_monografie.get(autor_id)

            pkt_monografie, prace_monografie = policz_knapsack(
                monografie,
                maks_slot=maksymalny_slot_za_monografie,
            )
            potencjalny_slot_za_zdane_monografie = sum(x.slot for x in prace_monografie)

            pkt_nie_monografie_maks, prace_nie_monografie_maks = policz_knapsack(
                nie_monografie,
                maks_slot=maksymalny_slot_za_calosc,
            )

            pkt_nie_monografie_min, prace_nie_monografie_min = policz_knapsack(
                nie_monografie,
                maks_slot=maksymalny_slot_za_calosc
                - potencjalny_slot_za_zdane_monografie,
            )

            if pkt_nie_monografie_maks > (pkt_nie_monografie_min + pkt_monografie):
                [self.zsumuj_pojedyncza_prace(x) for x in prace_nie_monografie_maks]
            else:
                [self.zsumuj_pojedyncza_prace(x) for x in prace_monografie]
                [self.zsumuj_pojedyncza_prace(x) for x in prace_nie_monografie_min]



# Generated by Django 3.0.7 on 2020-07-27 21:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0218_auto_20200727_2307"),
    ]

    operations = [
        migrations.AlterField(
            model_name="grant",
            name="rok",
            field=models.PositiveSmallIntegerField(blank=True, null=True),
        ),
    ]



# Generated by Django 3.2.15 on 2022-10-09 14:10

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0330_merge_20220921_2152"),
    ]

    operations = [
        migrations.AddField(
            model_name="jednostka",
            name="rodzaj_jednostki",
            field=models.CharField(
                choices=[
                    (
                        "normalna",
                        "zwyczajna jednostka (katedra, zakład, pracownia, itp.)",
                    ),
                    ("kolo_naukowe", "koło naukowe"),
                ],
                db_index=True,
                default="normalna",
                max_length=20,
            ),
        ),
        migrations.CreateModel(
            name="Kierunek_Studiow",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("nazwa", models.CharField(max_length=512, unique=True)),
                ("skrot", models.CharField(max_length=128, unique=True)),
                ("opis", models.TextField(blank=True, null=True)),
                ("adnotacje", models.TextField(blank=True, null=True)),
                (
                    "wydzial",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT, to="bpp.wydzial"
                    ),
                ),
            ],
            options={
                "verbose_name": "kierunek studiów",
                "verbose_name_plural": "kierunki studiów",
                "ordering": ("nazwa",),
            },
        ),
        migrations.AddField(
            model_name="patent_autor",
            name="kierunek_studiow",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="bpp.kierunek_studiow",
            ),
        ),
        migrations.AddField(
            model_name="wydawnictwo_ciagle_autor",
            name="kierunek_studiow",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="bpp.kierunek_studiow",
            ),
        ),
        migrations.AddField(
            model_name="wydawnictwo_zwarte_autor",
            name="kierunek_studiow",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="bpp.kierunek_studiow",
            ),
        ),
    ]



from pathlib import Path

import pytest


@pytest.fixture
def parent_path():
    return Path(__file__).parent


@pytest.fixture
def test1_xlsx(parent_path):
    return str(parent_path / "test1.xlsx")


@pytest.fixture
def default_xlsx(parent_path):
    return str(parent_path / "default.xlsx")


@pytest.fixture
def test2_bad_header_xlsx(parent_path):
    return str(parent_path / "test2_bad_header.xlsx")



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-16 11:13
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0128_zewnetrzne_bazy_view'),
    ]

    operations = [
        migrations.CreateModel(
            name='ZewnetrzneBazyDanychView',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('info', models.TextField()),
            ],
            options={
                'db_table': 'bpp_zewnetrzne_bazy_view',
                'managed': False,
            },
        ),
    ]



# -*- coding: utf-8 -*-


from django.db import migrations, models
from datetime import datetime

class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0040_auto_20160802_2209'),
    ]

    operations = [
        migrations.AddField(
            model_name='wydawnictwo_ciagle',
            name='ostatnio_zmieniony_dla_pbn',
            field=models.DateTimeField(default='2001-01-01 00:00', auto_now_add=True, help_text=b'Ostatnia aktualizacja rekordu - zmiana dowolnego z p\xc3\xb3l z awyj\xc4\x85tkiem blok\xc3\xb3w p\xc3\xb3l: \xe2\x80\x9epunktacja\xe2\x80\x9d,\n        \xe2\x80\x9epunktacja komisji centralnej\xe2\x80\x9d, \xe2\x80\x9eadnotacje\xe2\x80\x9d oraz pole \xe2\x80\x9estatus korekty\xe2\x80\x9d.', verbose_name=b'Ostatnio zmieniony (dla PBN)'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='wydawnictwo_zwarte',
            name='ostatnio_zmieniony_dla_pbn',
            field=models.DateTimeField(default='2001-01-01 00:00', auto_now_add=True, help_text=b'Ostatnia aktualizacja rekordu - zmiana dowolnego z p\xc3\xb3l z awyj\xc4\x85tkiem blok\xc3\xb3w p\xc3\xb3l: \xe2\x80\x9epunktacja\xe2\x80\x9d,\n        \xe2\x80\x9epunktacja komisji centralnej\xe2\x80\x9d, \xe2\x80\x9eadnotacje\xe2\x80\x9d oraz pole \xe2\x80\x9estatus korekty\xe2\x80\x9d.', verbose_name=b'Ostatnio zmieniony (dla PBN)'),
            preserve_default=False,
        ),
        migrations.RunSQL("UPDATE bpp_wydawnictwo_ciagle SET ostatnio_zmieniony_dla_pbn = ostatnio_zmieniony"),
        migrations.RunSQL("UPDATE bpp_wydawnictwo_zwarte SET ostatnio_zmieniony_dla_pbn = ostatnio_zmieniony"),
    ]



from django.db import models

from .base import BasePBNMongoDBModel

from django.utils.functional import cached_property

from bpp import const
from bpp.models import LinkDoPBNMixin


class Scientist(LinkDoPBNMixin, BasePBNMongoDBModel):
    url_do_pbn = const.LINK_PBN_DO_AUTORA
    atrybut_dla_url_do_pbn = "pk"

    from_institution_api = models.BooleanField(
        "Rekord z API instytucji", db_index=True, null=True
    )

    lastName = models.TextField(db_index=True, null=True, blank=True)
    name = models.TextField(db_index=True, null=True, blank=True)
    pbnId = models.TextField(db_index=True, null=True, blank=True)
    qualifications = models.TextField("Tytuł", db_index=True, null=True, blank=True)
    orcid = models.TextField(db_index=True, null=True, blank=True)
    polonUid = models.TextField(db_index=True, null=True, blank=True)

    pull_up_on_save = ["lastName", "name", "qualifications", "orcid", "polonUid"]

    class Meta:
        verbose_name = "Osoba w PBN API"
        verbose_name_plural = "Osoby w PBN API"

        unique_together = [
            "mongoId",
            "lastName",
            "name",
            "orcid",
        ]

    def currentEmploymentsInstitutionDisplayName(self):
        ces = self.value("object", "currentEmployments", return_none=True)
        if ces is not None:
            return ces[0].get("institutionDisplayName")

    def __str__(self):
        ret = (
            f"{self.lastName} {self.name}, {self.qualifications or '-'}, "
            f"{self.currentEmploymentsInstitutionDisplayName() or '-'}, (PBN ID: {self.pk})"
        )
        ret = ret.replace(" ,", ",")
        ret = ret.replace("-, ", "")
        ret = ret.replace(", (", " (")
        return ret

    @cached_property
    def rekord_w_bpp(self):
        from bpp.models.autor import Autor

        try:
            return Autor.objects.filter(pbn_uid_id=self.pk).first()
        except Autor.DoesNotExist:
            pass



# -*- coding: utf-8 -*-


from django.db import migrations, models
from django.db.migrations.operations.special import RunPython

from bpp.models.struktura import Jednostka

def zaktualizuj(apps, schema_editor):
    Wydzial = apps.get_model("bpp", "Wydzial")

    for elem in ['Brak wpisanego wydziału',
                 'Wydział Lekarski',
                 'Jednostki Dawne',
                 'Bez Wydziału',
                 'Poza Wydziałem']:
        try:
            elem = Wydzial.objects.get(nazwa=elem)
        except Wydzial.DoesNotExist:
            continue
        elem.wirtualny = True
        elem.save()

    try:
        elem = Wydzial.objects.get(nazwa='Jednostki Dawne')
        elem.archiwalny = True
        elem.save()
    except Wydzial.DoesNotExist:
        pass

    Jednostka = apps.get_model("bpp", "Jednostka")

    for elem in ['BŁĄD: Brak wpisanej jednostki.',
                 'Studia doktoranckie',
                 'Obca jednostka']:
        try:
            elem = Jednostka.objects.get(nazwa=elem)
        except Jednostka.DoesNotExist:
            continue
        elem.wirtualna = True
        if elem.nazwa == 'Obca jednostka':
            elem.obca_jednostka = True
        elem.save()


class Migration(migrations.Migration):
    dependencies = [
        ('bpp', '0043_auto_20160817_1151'),
    ]

    operations = [
        migrations.AddField(
            model_name='jednostka',
            name='nie_archiwizuj',
            field=models.BooleanField(default=False,
                                      help_text=b'Je\xc5\xbceli zaznaczono to pole, to przy imporcie danych\n    na temat struktury uczelni z zewn\xc4\x99trznych \xc5\xbar\xc3\xb3de\xc5\x82 ta jednostka nie b\xc4\x99dzie przenoszona do wydzia\xc5\x82u oznaczonego\n    jako archiwalny.'),
        ),

        RunPython(zaktualizuj)

    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-15 06:13
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('import_dyscyplin', '0007_auto_20180414_1917'),
    ]

    operations = [
        migrations.AddField(
            model_name='import_dyscyplin_row',
            name='imiona',
            field=models.CharField(default='', max_length=200),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='import_dyscyplin_row',
            name='nazwisko',
            field=models.CharField(default='', max_length=200),
            preserve_default=False,
        ),
    ]
